@@ -1141,8 +1141,14 @@ gdl_dock_item_dock (GdlDockObject    *object,
 {
     GdlDockObject *new_parent, *parent;
     gboolean       add_ourselves_first;
-    
+
+    guint	   available_space=0;
+    gint	   pref_size=-1;
+    guint	   splitpos=0;
+    GtkRequisition req;
+  	
     parent = gdl_dock_object_get_parent_object (object);
+    gdl_dock_item_preferred_size (GDL_DOCK_ITEM (requestor), &req);
 
     switch (position) {
         case GDL_DOCK_TOP:
@@ -1152,6 +1158,9 @@ gdl_dock_item_dock (GdlDockObject    *object,
                                        "orientation", GTK_ORIENTATION_VERTICAL,
                                        NULL);
             add_ourselves_first = (position == GDL_DOCK_BOTTOM);
+            if (parent)
+            	available_space=GTK_WIDGET(parent)->allocation.height;
+            pref_size = req.height;
             break;
         case GDL_DOCK_LEFT:
         case GDL_DOCK_RIGHT:
@@ -1159,6 +1168,9 @@ gdl_dock_item_dock (GdlDockObject    *object,
                                        "orientation", GTK_ORIENTATION_HORIZONTAL,
                                        NULL);
             add_ourselves_first = (position == GDL_DOCK_RIGHT);
+            if(parent)
+            	available_space = GTK_WIDGET(parent)->allocation.width;
+            pref_size = req.width;
             break;
         case GDL_DOCK_CENTER:
             new_parent = g_object_new (gdl_dock_object_type_from_nick ("notebook"),
@@ -1198,9 +1210,11 @@ gdl_dock_item_dock (GdlDockObject    *object,
     if (add_ourselves_first) {
         gtk_container_add (GTK_CONTAINER (new_parent), GTK_WIDGET (object));
         gtk_container_add (GTK_CONTAINER (new_parent), GTK_WIDGET (requestor));
+	splitpos = available_space - pref_size;
     } else {
         gtk_container_add (GTK_CONTAINER (new_parent), GTK_WIDGET (requestor));
         gtk_container_add (GTK_CONTAINER (new_parent), GTK_WIDGET (object));
+        splitpos = pref_size;
     }
 
     /* add the new parent to the parent */
@@ -1218,6 +1232,8 @@ gdl_dock_item_dock (GdlDockObject    *object,
         g_object_set (G_OBJECT (new_parent),
                       "position", g_value_get_uint (other_data),
                       NULL);
+    }else if (splitpos > 0 && splitpos < available_space) {
+			g_object_set (G_OBJECT (new_parent), "position", splitpos, NULL);
     }
     
     GDL_DOCK_OBJECT_UNSET_FLAGS (object, GDL_DOCK_IN_REFLOW);
@@ -1392,7 +1408,7 @@ gdl_dock_item_showhide_grip (GdlDockItem *item)
     
     if (item->_priv->grip) {
         if (GDL_DOCK_ITEM_GRIP_SHOWN (item) && 
-	    GDL_DOCK_ITEM_NOT_LOCKED(item))
+            GDL_DOCK_ITEM_NOT_LOCKED(item))
              cursor = gdk_cursor_new_for_display (display, GDK_HAND2);
     }
     if (item->_priv->grip && GDL_DOCK_ITEM_GRIP (item->_priv->grip)->title_window)
@@ -1466,10 +1482,11 @@ gdl_dock_item_dock_to (GdlDockItem      *item,
     g_return_if_fail (item != NULL);
     g_return_if_fail (item != target);
     g_return_if_fail (target != NULL || position == GDL_DOCK_FLOATING);
-    
+    g_return_if_fail ((item->behavior & GDL_DOCK_ITEM_BEH_NEVER_FLOATING) == 0 || position != GDL_DOCK_FLOATING);
+
     if (position == GDL_DOCK_FLOATING || !target) {
         GdlDockObject *controller;
-        
+
         if (!gdl_dock_object_is_bound (GDL_DOCK_OBJECT (item))) {
             g_warning (_("Attempt to bind an unbound item %p"), item);
             return;
@@ -1611,13 +1628,36 @@ gdl_dock_item_hide_item (GdlDockItem *item)
        restore the position later */
     if (!GDL_DOCK_OBJECT_AUTOMATIC (item)) {
         if (item->_priv->ph)
-            g_object_unref (item->_priv->ph);
+			g_object_unref (item->_priv->ph); 
         
+		gboolean isFloating = FALSE;
+		gint width=0, height=0, x=0, y = 0;
+		
+		
+		if( GDL_IS_DOCK (gdl_dock_object_get_parent_object (GDL_DOCK_OBJECT (item))))
+		{
+			GdlDock* dock = GDL_DOCK (gdl_dock_object_get_parent_object (GDL_DOCK_OBJECT (item)));
+			g_object_get (dock,
+						  "floating", &isFloating, 
+						  "width", &width,
+						  "height",&height,
+						  "floatx",&x,
+						  "floaty",&y,
+						  NULL);
+		}else{
+			item->_priv->preferred_width=GTK_WIDGET (item)->allocation.width;
+			item->_priv->preferred_height=GTK_WIDGET (item)->allocation.height;
+		}
         item->_priv->ph = GDL_DOCK_PLACEHOLDER (
             g_object_new (GDL_TYPE_DOCK_PLACEHOLDER,
                           "sticky", FALSE,
                           "host", item,
-                          NULL));
+                          "width", width,
+                          "height", height,
+						  "floating", isFloating,
+						  "floatx", x,
+						  "floaty", y,
+						  NULL));
         g_object_ref (item->_priv->ph);
         gtk_object_sink (GTK_OBJECT (item->_priv->ph));
     }
@@ -1634,7 +1674,7 @@ gdl_dock_item_hide_item (GdlDockItem *item)
     gdl_dock_object_detach (GDL_DOCK_OBJECT (item), TRUE);
 
     gtk_widget_hide (GTK_WIDGET (item));
-	
+
     gdl_dock_object_thaw (GDL_DOCK_OBJECT (item));
 }
 
@@ -1653,20 +1693,48 @@ gdl_dock_item_show_item (GdlDockItem *item)
     g_return_if_fail (item != NULL);
 
     GDL_DOCK_OBJECT_UNSET_FLAGS (item, GDL_DOCK_ICONIFIED);
-
+			
     if (item->_priv->ph) {
-        gtk_container_add (GTK_CONTAINER (item->_priv->ph), GTK_WIDGET (item));
+		gboolean isFloating=FALSE;
+		gint width = 0, height = 0, x= 0, y = 0;
+		g_object_get (G_OBJECT(item->_priv->ph),
+					  "width", &width,
+					  "height", &height,
+					  "floating",&isFloating,
+					  "floatx", &x,
+					  "floaty", &y,
+					  NULL);
+		if (isFloating) {
+			GdlDockObject *controller = gdl_dock_master_get_controller (GDL_DOCK_OBJECT_GET_MASTER (item));
+			gdl_dock_add_floating_item (GDL_DOCK (controller),
+										item, x, y, width, height);
+        }else{
+			gtk_container_add (GTK_CONTAINER (item->_priv->ph), GTK_WIDGET (item));
+		}
         g_object_unref (item->_priv->ph);
         item->_priv->ph = NULL;
     }
     else if (gdl_dock_object_is_bound (GDL_DOCK_OBJECT (item))) {
-        GdlDockObject *toplevel = gdl_dock_master_get_controller (
-            GDL_DOCK_OBJECT_GET_MASTER (item));
-        if (toplevel) {
+        GdlDockObject *toplevel;
+
+	toplevel = gdl_dock_master_get_controller (
+	            GDL_DOCK_OBJECT_GET_MASTER (item));
+ 
+	if (item->behavior & GDL_DOCK_ITEM_BEH_NEVER_FLOATING) {
+	    g_warning("Object %s has no default position and flag GDL_DOCK_ITEM_BEH_NEVER_FLOATING is set.\n",
+			GDL_DOCK_OBJECT(item)->name);
+	}
+	else if(toplevel) {
             gdl_dock_object_dock (toplevel, GDL_DOCK_OBJECT (item),
                                   GDL_DOCK_FLOATING, NULL);
         }
+	else
+	    g_warning("There is no toplevel window. GdlDockItem %s cannot be shown.\n", GDL_DOCK_OBJECT(item)->name);
+
     }
+    else
+	g_warning("GdlDockItem %s is not bound. It cannot be shown.\n", GDL_DOCK_OBJECT(item)->name);
+
     gtk_widget_show (GTK_WIDGET (item));
 }
 
