@@ -6,20 +6,19 @@
  *
  * Copyright (C) 2002 Gustavo Girldez <gustavo.giraldez@gmx.net>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.  
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +26,6 @@
 #endif
 
 #include "gdl-i18n.h"
-#include <gdk/gdkx.h>
 
 #include "gdl-tools.h"
 #include "gdl-dock-master.h"
@@ -306,7 +304,7 @@ foreach_lock_unlock (GdlDockItem *item,
     if (gdl_dock_object_is_compound (GDL_DOCK_OBJECT (item)))
         gtk_container_foreach (GTK_CONTAINER (item),
                                (GtkCallback) foreach_lock_unlock,
-                               (gpointer) locked);
+                               GINT_TO_POINTER (locked));
 }
 
 static void
@@ -324,7 +322,7 @@ gdl_dock_master_lock_unlock (GdlDockMaster *master,
     /* just to be sure hidden items are set too */
     gdl_dock_master_foreach (master,
                              (GFunc) foreach_lock_unlock,
-                             (gpointer) locked);
+                             GINT_TO_POINTER (locked));
 }
 
 static void
@@ -492,7 +490,7 @@ gdl_dock_master_drag_motion (GdlDockItem *item,
     }
     else {
         GList *l;
-        
+
         /* try to dock the item in all the docks in the ring in turn */
         for (l = master->toplevel_docks; l; l = l->next) {
             dock = GDL_DOCK (l->data);
@@ -507,10 +505,15 @@ gdl_dock_master_drag_motion (GdlDockItem *item,
                 break;
         }
     }
-    
+
+  
     if (!may_dock) {
         GtkRequisition req;
-        
+	/* Special case for GdlDockItems : they must respect the flags */
+	if(GDL_IS_DOCK_ITEM(item)
+	&& GDL_DOCK_ITEM(item)->behavior & GDL_DOCK_ITEM_BEH_NEVER_FLOATING)
+	    return;
+
         dock = NULL;
         my_request.target = GDL_DOCK_OBJECT (
             gdl_dock_object_get_toplevel (request->applicant));
@@ -529,7 +532,15 @@ gdl_dock_master_drag_motion (GdlDockItem *item,
 
         g_value_init (&my_request.extra, GDK_TYPE_RECTANGLE);
         g_value_set_boxed (&my_request.extra, &my_request.rect);
-    };
+    }
+    /* if we want to enforce GDL_DOCK_ITEM_BEH_NEVER_FLOATING		*/
+    /* the item must remain attached to the controller, otherwise	*/
+    /* it could be inserted in another floating dock			*/
+    /* so check for the flag at this moment				*/
+    else if(GDL_IS_DOCK_ITEM(item)
+	&& GDL_DOCK_ITEM(item)->behavior & GDL_DOCK_ITEM_BEH_NEVER_FLOATING
+	&& dock != GDL_DOCK(master->controller))
+	    return;
 
     if (!(my_request.rect.x == request->rect.x &&
           my_request.rect.y == request->rect.y &&
@@ -583,7 +594,7 @@ gdl_dock_master_xor_rect (GdlDockMaster *master)
     }
     
     rect = &master->_priv->drag_request->rect;
-    window = gdk_window_lookup (gdk_x11_get_default_root_xwindow ());
+    window = gdk_get_default_root_window ();
 
     if (!master->_priv->root_xor_gc) {
         GdkGCValues values;
